@@ -2,11 +2,9 @@
                           cmudview.cpp
                       -------------------
     description		 : class providing a colored output view
-    begin		       : Wed Sep 8 1999
-    copyright		   : (C) 1999 by Stephan Uhlmann,
-                                 Andre Alexander Bell
-    email	         : suhlmann@gmx.de
-                     andre.bell@gmx.de
+    begin		 : Wed Sep 8 1999
+    copyright            : (C) 1999 by Kmud Developer Team
+    email                : kmud-devel@kmud.de
  ***************************************************************************/
 
 /***************************************************************************
@@ -25,6 +23,7 @@
 #include <qpixmap.h>
 #include <qregexp.h>
 #include <qfontmetrics.h>
+#include <qscrollbar.h>
 
 // for copy&paste
 #include <qclipboard.h>
@@ -33,8 +32,6 @@
 #include <kapp.h>
 
 
-#include <iostream.h>
-
 
 CMudView::CMudView(KmudDoc* document, QWidget * parent, const char * name)
  : QTableView(parent,name)
@@ -43,13 +40,15 @@ CMudView::CMudView(KmudDoc* document, QWidget * parent, const char * name)
 
 	myParent = parent;
 
+	fontMetric=NULL;
+
 	init();
 
 	setMaxLines(5000);
 
-	setForeground(QColor(0,0,0));
-	setBackground(QColor(255,255,255));
-	setFont(QFont("fixed"));
+	setDefaultForeground(QColor(0,0,0));
+	setDefaultBackground(QColor(255,255,255));
+	setDefaultFont(QFont("fixed"));
 
 	// for cut&paste
 	markedAreaStartRow = -1;
@@ -77,7 +76,6 @@ void CMudView::init()
 	setAutoUpdate(false);
 
 	setTopLeftCell(0,0);
-	lineX=0;
 
 	data.setAutoDelete(true);
 	data.clear();
@@ -92,14 +90,12 @@ void CMudView::init()
 	setNumRows(1);
 	setNumCols(1);
 	updateTableSize();
-
-	fontMetric=NULL;
+	currentMaxWidth = viewWidth();
 
 	setBackgroundColor(defaultBackground);
-	setTableFlags( Tbl_autoVScrollBar );
+	setTableFlags( Tbl_autoScrollBars );
 	setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
 	setCellHeight(ld->lineHeight);
-	setCellWidth(viewWidth());
 
 	vSliderAtBottom=false;
 
@@ -182,27 +178,47 @@ void CMudView::paintCell(QPainter* painter, int row, int col )
 	painter->drawPixmap(rect.topLeft(), *buf, rect);
 
 	delete buf;
+
+	if (x > currentMaxWidth)
+	{
+	  currentMaxWidth = x;
+	}
 }
 
 void CMudView::resizeEvent(QResizeEvent* e)
 {
+//  currentMaxWidth = viewWidth();
 	QTableView::resizeEvent(e);
-	setCellWidth(viewWidth());
 }
 
-QString CMudView::append(QString s, QColor foreground, QColor background, QFont font)
+void CMudView::createNewLine()
 {
 	MudViewLineData* ld;
+
+	ld = new MudViewLineData;
+	(ld->chunks).setAutoDelete(true);
+	ld->lineHeight=0;
+	data.append(ld);
+	setNumRows(numRows()+1);
+	updateTableSize();
+}
+
+
+QString CMudView::append(QString s, QColor foreground, QColor background, QFont font, bool noANSIColors)
+{
 	QString ansiAttrib[8];
 
 	// t is important (thats why it has the descibing name "t" ;-)
 	// we iterate through s char by char and build up t
 	// t is then flushed to the current chunk occasionally
 	QString t="";
+	// i is also important (as all variables without a selfexplaining name)
+	// it is the iterator while going over s char by char and creating t
+	int i=0;
 
 	QString ansiCode, ansiCodeChunk, ansiCodeAttrib,u,real,tabSpace;
 
-	int tabSpaceI,i=0,l=0,ansiStart=0,ansiI,lastAttrib=0,count=0;
+	int tabSpaceI,l=0,ansiStart=0,ansiI,lastAttrib=0,count=0;
 	//int ansiEnd=0;
 	char c, ansiCommand = 'm';
 	bool useBold=false, useBright=false, colorUsed=false, useNormal=true;
@@ -213,8 +229,9 @@ QString CMudView::append(QString s, QColor foreground, QColor background, QFont
 	
 	bool scrollDown=(yOffset()==maxYOffset());
 
-	QString newWord = QString("");
-	QString oldLine = QString("");
+	QString currentLine="";
+	QString nextWord = "";
+	int nextWordEndPos;
 
 	// the string we return, cleared from ANSI codes and certain special chars
 	// used for logging
@@ -222,9 +239,9 @@ QString CMudView::append(QString s, QColor foreground, QColor background, QFont
 
 
 
-
 	setAutoUpdate(false); // avoid flickering
-	for (i=0;i<(int)s.size();i++) {
+
+	for (i=0;i<=(int)s.length();i++) {
 		c=s.at(i);
 		switch (c) {
 			case '\007' :	
@@ -235,30 +252,19 @@ QString CMudView::append(QString s, QColor foreground, QColor background, QFont
 
 			case '\t'   : // expand tabs to spaces
 				tabSpace="";
-				for (tabSpaceI=0;tabSpaceI<(8-(lineX&7));tabSpaceI++)
+				for (tabSpaceI=0;tabSpaceI<(int)(8-(currentLine.length()&7));tabSpaceI++)
 					tabSpace+=" ";
 				s.replace(i,1,tabSpace);
-				i--;lineX--;
+				i--;
 
 				break;
 
 			case '\n'   : // flush the chunk
-				returnString+=t+"\n";
+				t+=c;
+				returnString+=t;
 				flushChunk(t,foreground,background,font);
-
-				//and make a new line
-				ld = new MudViewLineData;
-				(ld->chunks).setAutoDelete(true);
-				ld->lineHeight=0;
-				data.append(ld);
-				setNumRows(numRows()+1);
-				updateTableSize();
-
-				oldLine.setStr("");
-				newWord.setStr("");
-				lineX=0;
-
-
+				createNewLine();
+				currentLine="";
 				break;
 
 
@@ -319,9 +325,9 @@ QString CMudView::append(QString s, QColor foreground, QColor background, QFont
 					case 0: /** DEFAULT -- detects ANSI/VT100 for you */
 					case 1: /** ANSI -- Force ANSI -- Combined 0 and 1 because they're so similar */
 
-                        /** \x1B[2J Erases the screen with the background color and moves the cursor home.
-                         ** According to ANSI Standard, that sequence "Clears screen and homes cursor".
-                         **/
+						/** \x1B[2J Erases the screen with the background color and moves the cursor home.
+						** According to ANSI Standard, that sequence "Clears screen and homes cursor".
+						**/
 						if ((s.at(ansiStart + 1) == '[') && (s.at(ansiStart + 2) == '2') && (s.at(ansiStart + 3) == 'J')){
 							erase();
 							init();
@@ -337,7 +343,8 @@ QString CMudView::append(QString s, QColor foreground, QColor background, QFont
 								useNormal=false;
 								switch (ansiAttrib[count].toInt()) {
 									case 0:
-										actualBackground = defaultBackground;
+										if (noANSIColors==false)
+											actualBackground = defaultBackground;
 										useBold = false;
 										useBright = false;
 										useUnderline = false;
@@ -368,14 +375,16 @@ QString CMudView::append(QString s, QColor foreground, QColor background, QFont
 										/** Insert blinking code here */
 									break;
 									case 7:
-										actualBackground = defaultForeground;
+										if (noANSIColors==false)
+											actualBackground = defaultForeground;
 									case 8:
-										actualForeground = defaultBackground;
+										if (noANSIColors==false)
+											actualForeground = defaultBackground;
 								}
 							}
 							//forground colors
 							else if ((ansiAttrib[count].toInt() >= 30) && (ansiAttrib[count].toInt() <= 37)){
-								actualForeground = defaultForeground;
+//								actualForeground = defaultForeground;
 								colorUsed = true;
 								useNormal = false;
 								actualFont.setBold(false);
@@ -402,47 +411,57 @@ QString CMudView::append(QString s, QColor foreground, QColor background, QFont
 							
 							/** Background colors */
 							else if ((ansiAttrib[count].toInt() >= 40) && (ansiAttrib[count].toInt() <= 47)) {
-								switch (ansiAttrib[count].toInt()) {
-									case 40: actualBackground = FBgColor[BG_BLACK]; break;
-									case 41: actualBackground = FBgColor[BG_RED]; break;
-									case 42: actualBackground = FBgColor[BG_GREEN]; break;
-									case 43: actualBackground = FBgColor[BG_YELLOW]; break;
-									case 44: actualBackground = FBgColor[BG_BLUE]; break;
-									case 45: actualBackground = FBgColor[BG_MAGENTA]; break;
-									case 46: actualBackground = FBgColor[BG_CYAN]; break;
-									case 47: actualBackground = FBgColor[BG_WHITE]; break;
+								if (noANSIColors==false)
+								{
+									switch (ansiAttrib[count].toInt()) {
+										case 40: actualBackground = FBgColor[BG_BLACK]; break;
+										case 41: actualBackground = FBgColor[BG_RED]; break;
+										case 42: actualBackground = FBgColor[BG_GREEN]; break;
+										case 43: actualBackground = FBgColor[BG_YELLOW]; break;
+										case 44: actualBackground = FBgColor[BG_BLUE]; break;
+										case 45: actualBackground = FBgColor[BG_MAGENTA]; break;
+										case 46: actualBackground = FBgColor[BG_CYAN]; break;
+										case 47: actualBackground = FBgColor[BG_WHITE]; break;
+									}
 								}
 							}
 							count++;
 						}
 
 						actualFont.setBold((!useBright)&&(useBold));
-						if (useNormal == true) {
-							actualForeground=defaultForeground;
-							actualBackground=defaultBackground;
-							actualFont=defaultFont;
-						}
-						if (useUnderline == true)
+						if (noANSIColors==false)
+							if (useNormal == true) {
+								actualForeground=defaultForeground;
+								actualBackground=defaultBackground;
+								actualFont=defaultFont;
+							}
+
+						if (useUnderline == false)
+							actualFont.setUnderline(false);
+						else
 							actualFont.setUnderline(true);
-						
-						if (useBright){
-							if (useBlack) actualForeground = FANSIColor[GRAY];
-							else if (useRed) actualForeground = FANSIColor[LIGHT_RED];
-							else if (useGreen) actualForeground = FANSIColor[LIGHT_GREEN];
-							else if (useYellow) actualForeground = FANSIColor[LIGHT_YELLOW];
-							else if (useBlue) actualForeground = FANSIColor[LIGHT_BLUE];
-							else if (useMagenta) actualForeground = FANSIColor[LIGHT_MAGENTA];
-							else if (useCyan) actualForeground = FANSIColor[LIGHT_CYAN];
-							else if (useWhite) actualForeground = FANSIColor[LIGHT_WHITE];
-						} else {
-							if (useBlack) actualForeground = FANSIColor[BLACK];
-							else if (useRed) actualForeground = FANSIColor[DARK_RED];
-							else if (useGreen) actualForeground = FANSIColor[DARK_GREEN];
-							else if (useYellow) actualForeground = FANSIColor[DARK_YELLOW];
-							else if (useBlue) actualForeground = FANSIColor[DARK_BLUE];
-							else if (useMagenta) actualForeground = FANSIColor[DARK_MAGENTA];
-							else if (useCyan) actualForeground = FANSIColor[DARK_CYAN];
-							else if (useWhite) actualForeground = FANSIColor[DARK_WHITE];
+
+						if (noANSIColors==false)
+						{						
+							if (useBright){
+								if (useBlack) actualForeground = FANSIColor[GRAY];
+								else if (useRed) actualForeground = FANSIColor[LIGHT_RED];
+								else if (useGreen) actualForeground = FANSIColor[LIGHT_GREEN];
+								else if (useYellow) actualForeground = FANSIColor[LIGHT_YELLOW];
+								else if (useBlue) actualForeground = FANSIColor[LIGHT_BLUE];
+								else if (useMagenta) actualForeground = FANSIColor[LIGHT_MAGENTA];
+								else if (useCyan) actualForeground = FANSIColor[LIGHT_CYAN];
+								else if (useWhite) actualForeground = FANSIColor[LIGHT_WHITE];
+							} else {
+								if (useBlack) actualForeground = FANSIColor[BLACK];
+								else if (useRed) actualForeground = FANSIColor[DARK_RED];
+								else if (useGreen) actualForeground = FANSIColor[DARK_GREEN];
+								else if (useYellow) actualForeground = FANSIColor[DARK_YELLOW];
+								else if (useBlue) actualForeground = FANSIColor[DARK_BLUE];
+								else if (useMagenta) actualForeground = FANSIColor[DARK_MAGENTA];
+								else if (useCyan) actualForeground = FANSIColor[DARK_CYAN];
+								else if (useWhite) actualForeground = FANSIColor[DARK_WHITE];
+							}
 						}
 						break;
 					case 2: /** VT100 -- forces VT100 interpretation */
@@ -483,59 +502,36 @@ QString CMudView::append(QString s, QColor foreground, QColor background, QFont
 
 						v->s=ansiCodeChunk;
 						(data.last())->chunks.append(v);
+
 						break;
 					}
-					// An escape character must not be counted for the wrapping
-					lineX--;
 					break;
 
 			// case '\x1B' ends
 
-			case ' '    : 
+			case ' '    :
 				t+=c;
-				newWord += c;
-				if ( fontMetric && (fontMetric->width(oldLine,-1)+fontMetric->width(newWord,-1)) > (myParent->width()-72) ){
-				// remove the last word from t and put in on the next line
-					if( t.length() > newWord.length() ) {
-						t = t.left( ( t.length() - newWord.length() ) );
-						returnString+=t;
- 						flushChunk(t,foreground,background,font);
-						// new line
-						ld = new MudViewLineData;
-						(ld->chunks).setAutoDelete(true);
-						ld->lineHeight=0;
-						data.append(ld);
-						setNumRows(numRows()+1);
-						updateTableSize();
-						t = newWord;
-						lineX= t.length() -1;
-					} else {
-						returnString+=t;
-						flushChunk(t,foreground,background,font);
-						// new line
-						ld = new MudViewLineData;
-						(ld->chunks).setAutoDelete(true);
-						ld->lineHeight=0;
-						data.append(ld);
-						setNumRows(numRows()+1);
-						updateTableSize();
-						lineX=0;
-					}
-					oldLine.setStr("");
-					newWord.setStr("");
-
-				} else {
-					oldLine += newWord;
-					newWord.setStr("");
+				currentLine+=c;
+				nextWordEndPos=s.find(QRegExp("\\s"),i+1);
+				if (nextWordEndPos>-1)
+				{
+					nextWord=s.mid(i+1,nextWordEndPos-i);
+					if (fontMetric)
+						if ( fontMetric->width(currentLine)+fontMetric->width(stripANSI(nextWord)) > viewWidth() )
+						{
+							returnString+=t;
+							flushChunk(t,foreground,background,font);
+							createNewLine();
+							currentLine="";
+						}
 				}
-
 				break;
+
 			default:
 				t+=c;
-				newWord += c;
-		}
-		lineX++;
-	}
+				currentLine+=c;
+		} // switch (c)
+	} // for (i=0;i<=(int)s.length();i++)
 
 
 	if (!t.isEmpty())
@@ -573,27 +569,95 @@ int CMudView::cellHeight(int row)
 	return (data.at(row)->lineHeight);
 }
 
-void CMudView::setForeground(QColor fgc)
+int CMudView::cellWidth(int col)
 {
-	defaultForeground=actualForeground=fgc;
-	repaint(false);
+  return currentMaxWidth;
 }
 
-void CMudView::setBackground(QColor bgc)
+
+QColor CMudView::getDefaultForeground()
+{
+	return defaultForeground;
+}
+
+QColor CMudView::getDefaultBackground()
+{
+	return defaultBackground;
+}
+
+QFont CMudView::getDefaultFont()
+{
+	return defaultFont;
+}
+
+
+void CMudView::setDefaultForeground(QColor fgc)
+{
+	defaultForeground=actualForeground=fgc;
+// not needed because change affects only the following text
+//	repaint(false);
+}
+
+void CMudView::setDefaultBackground(QColor bgc)
 {
 	defaultBackground=actualBackground=bgc;
 	setBackgroundColor(bgc);
-	repaint(false);
+// not needed because change affects only the following text
+//	repaint(false);
 }
 
-void CMudView::setFont(QFont font)
+void CMudView::setDefaultFont(QFont font)
 {
 	defaultFont=actualFont=font;
 	if (fontMetric!=NULL) delete fontMetric;
 	fontMetric = new QFontMetrics( font );
-	repaint(false);
+// not needed because change affects only the following text
+//	repaint(false);
 }
 
+
+QColor CMudView::getActualForeground()
+{
+	return actualForeground;
+}
+
+QColor CMudView::getActualBackground()
+{
+	return actualBackground;
+}
+
+QFont CMudView::getActualFont()
+{
+	return actualFont;
+}
+
+
+void CMudView::setActualForeground(QColor fgc)
+{
+	actualForeground=fgc;
+// not needed because change affects only the following text
+//	repaint(false);
+}
+
+void CMudView::setActualBackground(QColor bgc)
+{
+	actualBackground=bgc;
+//	setBackgroundColor(bgc);
+// not needed because change affects only the following text
+//	repaint(false);
+}
+
+void CMudView::setActualFont(QFont font)
+{
+	actualFont=font;
+	if (fontMetric!=NULL) delete fontMetric;
+	fontMetric = new QFontMetrics( font );
+// not needed because change affects only the following text
+//	repaint(false);
+}
+
+
+
 void CMudView::setInterpretType(int i)
 {
 	interpretType = i;
@@ -618,20 +682,6 @@ void CMudView::setOutputBgColor(int BgColor, QColor color)
   FBgColor[BgColor]=color;
 }
 
-QColor CMudView::getForeground()
-{
-	return defaultForeground;
-}
-
-QColor CMudView::getBackground()
-{
-	return defaultBackground;
-}
-
-QFont CMudView::getFont()
-{
-	return defaultFont;
-}
 
 int CMudView::getInterpretType()
 {
@@ -946,20 +996,41 @@ void CMudView::slotVSliderMoved(int i)
 	}
 }
 
+
 int CMudView::getVScrollBarWidth()
 {
-	return ((QWidget*)(verticalScrollBar()))->width();
+	int size=0;
+  if (verticalScrollBar()->isVisible())
+		size=verticalScrollBar()->width();
+	return size;
 }
 
-void CMudView::setVScrollBarVisible(bool visible)
+void CMudView::setScrollBarsVisible(bool visible)
 {
-	if (visible) {
+	if (visible)
+	{
 		setTableFlags( Tbl_autoVScrollBar );
-		return;
+		setTableFlags( Tbl_autoHScrollBar );
+	}
+	else
+	{
+		clearTableFlags( Tbl_autoVScrollBar );
+		clearTableFlags( Tbl_vScrollBar );
+		clearTableFlags( Tbl_autoHScrollBar );
+		clearTableFlags( Tbl_hScrollBar );
 	}
-	clearTableFlags( Tbl_autoVScrollBar );
 }
 
+int CMudView::getHScrollBarHeight()
+{
+	int size=0;
+  if (horizontalScrollBar()->isVisible())
+		size=horizontalScrollBar()->height();
+	return size;
+}
+
+
+
 /** overwrites the mouseDoubleClickEvent inherited from QWidget
 marks the word the mouse pints to */
 void CMudView::mouseDoubleClickEvent (QMouseEvent* e)
@@ -1063,3 +1134,37 @@ void CMudView::flushChunk(QString &t, QColor foreground, QColor background, QFon
 }
 
 
+QString CMudView::stripANSI(QString s)
+{
+	QString returnString="";
+	int i,j;
+
+	for (i=0;i<=(int)s.length();i++)
+	{
+		if (s.at(i)=='\x1B')
+		{
+			j=s.find(QRegExp("[a-zA-Z]"),i+1);
+			i=j;
+		}
+		else
+		returnString+=s.at(i);
+	}
+
+	return returnString;
+}
+
+int CMudView::translateIndexANSI(int index,QString s)
+{
+	int i,j,returnValue=index;
+
+	for (i=0;i<=returnValue;i++)
+	{
+		if (s.at(i)=='\x1B')
+		{
+			j=s.find(QRegExp("[a-zA-Z]"),i+1);
+			if (j>i) returnValue+=(j-i+1);
+			i=j;
+		}
+	}
+	return returnValue;
+}
