@@ -3,10 +3,8 @@
                       -------------------
     description          : Main Application Class
     begin                : Sun Jun 20 14:17:54 MEST 1999
-    copyright            : (C) 1999 by Stephan Uhlmann
-                                       Andre Alexander Bell
-    email                : suhlmann@gmx.de
-                           andre.bell@gmx.de
+    copyright            : (C) 1999 by Kmud Developer Team
+    email                : kmud-devel@kmud.de
  ***************************************************************************/
 
 /***************************************************************************
@@ -34,6 +32,7 @@
 #include "dialogs/dlgoptionmud.h"
 #include "dialogs/dlgmultilineinput.h"
 #include "dialogs/dlgpluginmanager.h"
+#include "dialogs/dlgcustomtoolbar.h"
 
 #include <kwm.h>
 
@@ -45,10 +44,10 @@ KmudApp::KmudApp()
 	app = (KmudApplication*)KApplication::getKApplication();
 	
 	// init member variables
-  docker = NULL;
-  accelerators = new KAccel(this);
+	docker = NULL;
+	accelerators = new KAccel(this);
 	
-
+	lastNumToolbarItems = -1;
 	///////////////////////////////////////////////////////////////////
 	// instatiate multiline input dialog, it's not modal so we just hide/show it
 	multiLineInput = new DlgMultiLineInput(NULL,"MultiLineInput");
@@ -60,7 +59,6 @@ KmudApp::KmudApp()
 	doc->setCurrentCharacterID("");
 	doc->setCurrentMudID("");
 
-
 	///////////////////////////////////////////////////////////////////
 	// read the config file options
 	// read them _before_ the inits so we init the right
@@ -70,19 +68,17 @@ KmudApp::KmudApp()
 
 	autoReconnect = false;
 
-#ifdef MAPPER
 	mapper = NULL;
-#endif
-
+	
 	///////////////////////////////////////////////////////////////////
 	// call inits to invoke all other construction parts
 	initMenuBar();
 	initToolBar();
 	initStatusBar();
+	refreshEnablesDisables();	
 	initView();
 	initTimer();
-  initAccelerators();
-	
+	initAccelerators();	
 	
 	///////////////////////////////////////////////////////////////////
 	// create new Telnet
@@ -113,9 +109,8 @@ KmudApp::KmudApp()
 	// create new Mud stream
 	mudstream = new CMudStream(this);
 
-#ifdef MAPPER
+
 	mudstream->setMapper(mapper);
-#endif
 	mudstream->setCharDBEntry(NULL);
 
 	connect (mudstream, SIGNAL(processedOutput()),this, SLOT(slotMudOutputReceived()) );
@@ -126,12 +121,16 @@ KmudApp::KmudApp()
 	// enable bars dependend on config file setups
 	if (!bViewToolbar)
 		enableToolBar(KToolBar::Hide,0);
+	if (!bViewCustomToolbar)
+		enableToolBar(KToolBar::Hide,1);
+			
 	if (!bViewStatusbar)
 		enableStatusBar(KStatusBar::Hide);
 
 
 	menuBar()->setMenuBarPos(menu_bar_pos);
-	toolBar()->setBarPos(tool_bar_pos);
+	mainToolBar->setBarPos(tool_bar_pos);
+	customToolBar->setBarPos(custom_tool_bar_pos);
 
 }
 
@@ -139,6 +138,8 @@ KmudApp::~KmudApp()
 {
 	// Make sure that is disable as the applications is about to quit
 	autoReconnect = false;
+	delete mainToolBar;
+	delete customToolBar;
 	delete mud;
 	delete mudstream;
 	delete mudlog;
@@ -178,19 +179,17 @@ CPluginManager* KmudApp::getPluginManager()
 	return plugin_manager;
 }
 
-#ifdef MAPPER
 CMapWindow* KmudApp::getMapper()
 {
 	return mapper;
 }
-#endif
 
 void KmudApp::enableCommand(int id_)
 {
 	///////////////////////////////////////////////////////////////////
 	// enable menu and toolbar functions by their ID's
 	menuBar()->setItemEnabled(id_,true);
-	toolBar()->setItemEnabled(id_,true);
+	mainToolBar->setItemEnabled(id_,true);
 }
 
 void KmudApp::disableCommand(int id_)
@@ -198,7 +197,7 @@ void KmudApp::disableCommand(int id_)
 	///////////////////////////////////////////////////////////////////
 	// disable menu and toolbar functions by their ID's
 	menuBar()->setItemEnabled(id_,false);
-	toolBar()->setItemEnabled(id_,false);
+	mainToolBar->setItemEnabled(id_,false);
 }
 
 void KmudApp::initTimer()
@@ -279,20 +278,19 @@ void KmudApp::initMenuBar()
 	// menuBar entry view_menu
 	view_menu = new QPopupMenu();
 	view_menu->setCheckable(true);
-	view_menu->insertItem(i18n("Tool&bar"), ID_VIEW_TOOLBAR);
+	view_menu->insertItem(i18n("Main Tool&bar"), ID_VIEW_TOOLBAR);
+	view_menu->insertItem(i18n("&Programmble Toolbar"), ID_VIEW_CUSTOM_TOOLBAR);	
 	view_menu->insertItem(i18n("&Statusbar"), ID_VIEW_STATUSBAR );
 	view_menu->insertItem(i18n("&Alternative Input "), ID_VIEW_ALTINPUT );
 	view_menu->insertItem(i18n("Multiline &Input "), ID_VIEW_MULTILINEINPUT );
-#ifdef MAPPER
 	view_menu->insertItem(i18n("Auto &Mapper"),ID_VIEW_MAPPER);
-#endif
 
 	view_menu->setItemChecked(ID_VIEW_TOOLBAR, bViewToolbar);
+	view_menu->setItemChecked(ID_VIEW_CUSTOM_TOOLBAR, bViewCustomToolbar);	
 	view_menu->setItemChecked(ID_VIEW_STATUSBAR, bViewStatusbar);
 	view_menu->setItemChecked(ID_VIEW_ALTINPUT, bViewAltInput);
 	view_menu->setItemChecked(ID_VIEW_MULTILINEINPUT, multiLineInput->isVisible());
 
-
 	///////////////////////////////////////////////////////////////////
 	// menuBar entry option_menu
 	option_menu = new QPopupMenu();
@@ -303,8 +301,6 @@ void KmudApp::initMenuBar()
 	option_menu->insertItem(i18n("&Preferences..."), ID_OPTION_PREFERENCES);
 	option_menu->insertSeparator();
 	mudOptionsID = option_menu->insertItem(i18n("Current &MUD Settings..."), ID_OPTION_MUD);
-	option_menu->insertSeparator();
-	option_menu->insertItem(i18n("&Save Options"), ID_OPTION_SAVE);
 
 	///////////////////////////////////////////////////////////////////
 	// menuBar entry profile_menu
@@ -313,6 +309,7 @@ void KmudApp::initMenuBar()
 	aliasID = profile_menu->insertItem(Icon("kmud_aliases.xpm"), i18n("&Aliases..."), ID_PROFILE_ALIASES);
 	triggerID = profile_menu->insertItem(Icon("kmud_triggers.xpm"), i18n("&Triggers..."), ID_PROFILE_TRIGGERS);
 	loggingID = profile_menu->insertItem(Icon("kmud_log.xpm"), i18n("&Logging..."), ID_PROFILE_LOGGING);
+	toolbarID = profile_menu->insertItem(i18n("&Toolbar..."),ID_PROFILE_TOOLBAR);
 	profile_menu->insertSeparator();
 	saveID = profile_menu->insertItem(i18n("&Save Current Profile"), ID_PROFILE_SAVE);
 
@@ -329,7 +326,7 @@ void KmudApp::initMenuBar()
 	QString aboutString="";
 	aboutString+=i18n("Kmud Version ");
 	aboutString+=VERSION;
-	aboutString+=i18n("\nCopyright (c) 1999, 2000 The Kmud Team <kmud-devel@kmud.de>\n    Stephan Uhlmann <suhlmann@gmx.de>\n    John-Paul Stanford <john-paul.stanford@virgin.net>\n    Roalt Zijlstra <roalt@dnd.utwente.nl>\n    Tomer Brisker <brisker@actcom.co.il>\n    Jrgen Kosche <mnementh@gmx.de>\n    Kevin Krammer <kevin.krammer@gmx.at>\n    Andre Alexander Bell <andre.bell@gmx.de>\nPlease see the AUTHORS file for all contributors.\n\nKmud Homepage: http://www.kmud.de\n\nKmud is distributed under the GNU General Public License.\nSee the file COPYING for further detail.");
+	aboutString+=i18n("\nCopyright (c) 1999,2000,2001 The Kmud Team <kmud-devel@kmud.de>\n    Stephan Uhlmann <stephan@kmud.de>\n    John-Paul Stanford <jp@stanwood.org.uk>\n    Roalt Zijlstra <roalt@dnd.utwente.nl>\n    Tomer Brisker <brisker@actcom.co.il>\n    Jrgen Kosche <mnementh@gmx.de>\n    Kevin Krammer <kevin.krammer@gmx.at>\n    Andre Alexander Bell <andre.bell@gmx.de>\nPlease see the AUTHORS file for all contributors.\n\nKmud Homepage: http://www.kmud.de\n\nKmud is distributed under the GNU General Public License.\nSee the file COPYING for further detail.");
 	help_menu = kapp->getHelpMenu(true, aboutString);
 	// create boss key entry
 	help_menu->insertSeparator();
@@ -357,8 +354,6 @@ void KmudApp::initMenuBar()
 	CONNECT_CMD(profile_menu);
 	CONNECT_CMD(plugin_menu);
 	CONNECT_CMD(help_menu);
-
-	refreshEnablesDisables();
 }
 
 void KmudApp::initToolBar()
@@ -367,34 +362,91 @@ void KmudApp::initToolBar()
 	///////////////////////////////////////////////////////////////////
 	// TOOLBAR
 	// set toolBar() the current toolBar and the position due to config file
-	toolBarConnectButtonPosition=toolBar()->insertButton(Icon("kmud_connectionwizard.xpm"), ID_MUD_CONNECTION_WIZARD, true, i18n("Connection Wizard"));
-	toolBar()->insertSeparator();
-	aliasToolID = toolBar()->insertButton(Icon("kmud_aliases.xpm"), ID_PROFILE_ALIASES, true, i18n("Aliases"));
-	triggerToolID = toolBar()->insertButton(Icon("kmud_triggers.xpm"), ID_PROFILE_TRIGGERS, true, i18n("Triggers"));
-	loggingToolID = toolBar()->insertButton(Icon("kmud_log.xpm"), ID_TOOLBAR_LOGGING_TOGGLE, true, i18n("Toggle Logging"));
-#ifdef MAPPER	
-	 mapperToolID = toolBar()->insertButton(Icon("kmud_mapper.xpm"), ID_VIEW_MAPPER, false, i18n("Show/Hide Mapper"));
-	// toolBar()->setToggle(ID_VIEW_MAPPER,true);
-#endif
-	toolBar()->insertSeparator();
-	toolBar()->insertButton(Icon("help.xpm"), ID_HELP, SIGNAL(pressed()), kapp, SLOT(appHelpActivated()), true, i18n("Help"));
-
-  ///////////////////////////////////////////////////////////////////
-  // INSERT YOUR APPLICATION SPECIFIC TOOLBARS HERE -e.g. tool_bar_1:
-  // add functionality for new created toolbars in:
-  // enableCommand, disableCommand, in the menuBar() and an additional function slotViewToolbar_1
-  // for that also create a bViewToolbar_1 and a KConfig entry (see Constructor).
-  // Also update resource values and commands 
+	mainToolBar = new KToolBar(this);
+	addToolBar(mainToolBar);
+	
+	toolBarConnectButtonPosition=mainToolBar->insertButton(Icon("kmud_connectionwizard.xpm"), ID_MUD_CONNECTION_WIZARD, true, i18n("Connection Wizard"));
+	mainToolBar->insertSeparator();
+	aliasToolID = mainToolBar->insertButton(Icon("kmud_aliases.xpm"), ID_PROFILE_ALIASES, true, i18n("Aliases"));
+	triggerToolID = mainToolBar->insertButton(Icon("kmud_triggers.xpm"), ID_PROFILE_TRIGGERS, true, i18n("Triggers"));
+	loggingToolID = mainToolBar->insertButton(Icon("kmud_log.xpm"), ID_TOOLBAR_LOGGING_TOGGLE, true, i18n("Toggle Logging"));
+	mapperToolID = mainToolBar->insertButton(Icon("kmud_mapper.xpm"), ID_VIEW_MAPPER, false, i18n("Show/Hide Mapper"));
 
+	mainToolBar->insertSeparator();
+	mainToolBar->insertButton(Icon("help.xpm"), ID_HELP, SIGNAL(pressed()), kapp, SLOT(appHelpActivated()), true, i18n("Help"));
 
+	customToolBar = new KToolBar(this);
+	addToolBar(customToolBar);
+	updateCustomToolbar();
+	
 	///////////////////////////////////////////////////////////////////
 	// CONNECT THE TOOLBAR SLOTS WITH SIGNALS - add new created toolbars
-	CONNECT_TOOLBAR(toolBar());
+	CONNECT_TOOLBAR(mainToolBar);
+	connect(customToolBar,SIGNAL(clicked(int)),this,SLOT(slotCustomToolbarClicked(int)));
 
 	refreshEnablesDisables();
 
 }
 
+/** Used to update the custom tool bar */
+void KmudApp::updateCustomToolbar(void)
+{
+	if (lastNumToolbarItems>-1)
+	{
+		for (int i=0; i<=lastNumToolbarItems;i++)
+		{
+			customToolBar->hideItem(i);
+			customToolBar->removeItem(i);
+		}		
+	}
+
+	CCharacterProfile* profile = doc->getCharacterProfile(doc->getCurrentCharacterID());
+
+	if (profile)
+	{
+		ToolbarEntry* entry = profile->firstToolbarEntry();
+		int count = 0;
+
+	 	while (entry !=NULL)
+	 	{
+	 		if (entry->icon == "")
+	 		{
+	 			QLabel *tmpIcon = new QLabel(this);
+	 			tmpIcon->setText(entry->name);
+	 			
+	 			QFont font("Helvetica",10);
+	 			font.setBold(false);
+			    font.setItalic(false);
+
+				QFontMetrics fm(font);
+				int w = fm.width(entry->name);
+//				int h = fm.height();
+	 			
+		 		QPixmap *pm = new QPixmap();
+		 		CCustomBarButton *button = new CCustomBarButton(*pm,count,customToolBar,"",w+5,(const char *)entry->name);
+		 		
+	 			customToolBar->insertWidget(count,button->width(),button);	 			
+	 			
+ 			    connect(button, SIGNAL(clicked(int)), this, SLOT(slotCustomToolbarClicked(int)));
+		 	}
+		 	else
+		 	{
+		 		customToolBar->insertButton(Icon("mini/"+entry->icon),count,true,entry->name);		 		
+		 	}
+		 	
+		 	
+		 	entry = profile->nextToolbarEntry();
+		 	count ++;
+	 	}
+	
+	
+		lastNumToolbarItems = count-1;	
+	}
+	else
+		lastNumToolbarItems = -1;	
+	
+}
+
 void KmudApp::initStatusBar()
 {
   	///////////////////////////////////////////////////////////////////
@@ -418,9 +470,9 @@ void KmudApp::initView()
 
 	///////////////////////////////////////////////////////////////////
 	// set view options as defined in readOptions()
-	view->setOutputForegroundColor(mudForegroundColor);
-	view->setOutputBackgroundColor(mudBackgroundColor);
-	view->setOutputFont(mudFont);
+	view->setOutputDefaultForegroundColor(mudForegroundColor);
+	view->setOutputDefaultBackgroundColor(mudBackgroundColor);
+	view->setOutputDefaultFont(mudFont);
 	view->setInputFont(inputFont);
 	view->setInputBackgroundColor(mudBackgroundColor);
 	view->setLocalEchoColor(mudLocalEchoColor);
@@ -440,6 +492,8 @@ void KmudApp::initView()
 	view->setAltInput(bViewAltInput);
 	view->setMaxHistoryLines(maxInputLines);
 	view->setMaxMudViewLines(maxOutputLines);
+	view->setSharedInputHistory(sharedInputHistory);
+	view->setEnableSpecialChars(enableSpecialChars);
 	view->setSeparator(separator);
 	view->setSpeedwalkingChar (speedwalkingChar);
 	view->setExternChar (externChar);
@@ -463,19 +517,21 @@ void KmudApp::saveOptions()
 	config->setGroup("APPEARANCE");
 	config->writeEntry("WindowWidth",width());
 	config->writeEntry("WindowHeight",height());
-	config->writeEntry("ShowToolbar",toolBar()->isVisible());
+	config->writeEntry("ShowToolbar",mainToolBar->isVisible());
+	config->writeEntry("ShowCustomToolbar",customToolBar->isVisible());	
 	config->writeEntry("ShowStatusbar",statusBar()->isVisible());
 	config->writeEntry("ShowAltInput",bViewAltInput);
 	config->writeEntry("MenuBarPos", (int)menuBar()->menuBarPos());
-	config->writeEntry("ToolBar_Pos", (int)toolBar()->barPos());
+	config->writeEntry("ToolBar_Pos", (int)mainToolBar->barPos());
+	config->writeEntry("CustomToolBar_Pos",(int)customToolBar->barPos());
 
 	config->setGroup("OPTIONS");
 
-	config->writeEntry("MudTextColor",view->getOutputForegroundColor());
+	config->writeEntry("MudTextColor",view->getOutputDefaultForegroundColor());
 
-	config->writeEntry("MudBackgroundColor",view->getOutputBackgroundColor());
+	config->writeEntry("MudBackgroundColor",view->getOutputDefaultBackgroundColor());
 
-	config->writeEntry("MudFont",view->getOutputFont());
+	config->writeEntry("MudFont",view->getOutputDefaultFont());
 
 	config->writeEntry("InputFont",view->getInputFont());
 
@@ -485,12 +541,15 @@ void KmudApp::saveOptions()
 
 	config->writeEntry("MaxOutputLines",maxOutputLines);
 	config->writeEntry("MaxInputLines",maxInputLines);
+	config->writeEntry("SharedInputHistory",sharedInputHistory);
 	config->writeEntry("Local_Echo",localEchoMode);
+	config->writeEntry("Local_Echo_SuppressNewline",localEchoSuppressNewline);
 	config->writeEntry("Input_Echo",inputEchoMode);
 	config->writeEntry("Keep_Last_input",keepLastInput);
 	config->writeEntry("Select_All_On_Focus",selectAllOnFocus);
 	config->writeEntry("AutoReconnect",autoReconnectOption);
 	config->writeEntry("AutoReconnectTime",autoTime);
+	config->writeEntry("EnableSpecialChars",enableSpecialChars);
 	config->writeEntry("CharSeparator",separator);
 	config->writeEntry("CharSpeedwalking", speedwalkingChar);
 	config->writeEntry("CharExternCommand", externChar);
@@ -599,6 +658,10 @@ void KmudApp::saveOptions()
 	
 	// save accelerators
 	accelerators->writeSettings(config);
+
+ 	config->setGroup("PING");
+	config->writeEntry("executable", doc->getPingExecutable());
+  config->writeEntry("enabled", doc->getPingEnabled());
 }
 
 void KmudApp::readOptions()
@@ -611,10 +674,12 @@ void KmudApp::readOptions()
 	mainWindowWidth = config->readNumEntry("WindowWidth",600);
 	mainWindowHeight = config->readNumEntry("WindowHeight",400);
 	bViewToolbar = config->readBoolEntry("ShowToolbar", true);
+	bViewCustomToolbar = config->readBoolEntry("ShowCustomToolbar", false);
 	bViewStatusbar = config->readBoolEntry("ShowStatusbar", true);
 	bViewAltInput = config->readBoolEntry("ShowAltInput", false);
 	menu_bar_pos = (KMenuBar::menuPosition)config->readNumEntry("MenuBarPos", KMenuBar::Top); 
 	tool_bar_pos = (KToolBar::BarPosition)config->readNumEntry("ToolBar_Pos", KToolBar::Top);
+	custom_tool_bar_pos = (KToolBar::BarPosition)config->readNumEntry("CustomToolBar_Pos", KToolBar::Top);
 
 
 	config->setGroup("OPTIONS");
@@ -633,8 +698,11 @@ void KmudApp::readOptions()
 	
 	maxOutputLines = config->readNumEntry("MaxOutputLines",5000);
 	maxInputLines = config->readNumEntry("MaxInputLines",1000);
+	sharedInputHistory = config->readBoolEntry("SharedInputHistory",false);
 	localEchoMode = config->readNumEntry("Local_Echo",2);
+	localEchoSuppressNewline = config->readBoolEntry("Local_Echo_SuppressNewline",false);
 	inputEchoMode = config->readNumEntry("Input_Echo",2);
+	enableSpecialChars = config->readBoolEntry("EnableSpecialChars",true);	
 	separator = config->readEntry("CharSeparator", ";");
 	speedwalkingChar = config->readEntry ("CharSpeedwalking", ".");
 	externChar = config->readEntry ("CharExternCommand", "#");
@@ -717,7 +785,10 @@ void KmudApp::readOptions()
 
 	config->setGroup("PLUGINS");
 	config->readListEntry("autoload",autoloadPlugins);
-	
+
+	config->setGroup("PING");
+	doc->setPingExecutable(config->readEntry("executable", "/bin/ping"));
+  doc->setPingEnabled(config->readBoolEntry("enabled", true));
 }
 
 /** Connect to a mud that as been setup in with the wizard
@@ -836,7 +907,7 @@ void KmudApp::connectToMud(QString mudTitle, QString login,QString mudHost,QStri
 	if (connectReturn>=0)
 	{
 		QObject::connect( mud, SIGNAL(filledBuffer(QString)), mudstream, SLOT(slotProcessOutput(QString)) );
-		// QObject::connect( view, SIGNAL(enteredInput(QString)), mudstream, SLOT(slotProcessInput(QString)) );
+		//QObject::connect( view, SIGNAL(enteredInput(QString)), mudstream, SLOT(slotProcessInput(QString)) );
 		QObject::connect( multiLineInput,SIGNAL(sendText(QString)),mudstream, SLOT(slotProcessInput(QString)) );
 		QObject::connect( mud, SIGNAL(changedEcho(bool)), view, SLOT(setLocalEcho(bool)) );
 		QObject::connect( mud, SIGNAL(changedEcho(bool)), view, SLOT(setInputEcho(bool)) );
@@ -937,8 +1008,8 @@ void KmudApp::connectToMud(QString mudTitle, QString login,QString mudHost,QStri
 
 
 		// update icons on toolbar
-		toolBar()->removeItem(ID_MUD_CONNECTION_WIZARD);
-		toolBar()->insertButton(Icon("kmud_disconnect.xpm"), ID_MUD_CLOSE, true, i18n("Close"),toolBarConnectButtonPosition);
+		mainToolBar->removeItem(ID_MUD_CONNECTION_WIZARD);
+		mainToolBar->insertButton(Icon("kmud_disconnect.xpm"), ID_MUD_CLOSE, true, i18n("Close"),toolBarConnectButtonPosition);
 
 		refreshLoggingToolbarIcon();
 
@@ -971,6 +1042,8 @@ void KmudApp::connectToMud(QString mudTitle, QString login,QString mudHost,QStri
 		mud_menu->setItemEnabled(ID_MUD_QUICKCONNECT2, false);
 		mud_menu->setItemEnabled(ID_MUD_QUICKCONNECT3, false);
 		mud_menu->setItemEnabled(ID_MUD_QUICKCONNECT4, false);
+		updateCustomToolbar();
+		
 	} // if (connectReturn>=0)
 
 	// enable/disable icons and menu slots on the current state of connection
@@ -1006,6 +1079,7 @@ void KmudApp::slotMudNewWindow()
 void KmudApp::slotMudConnectionWizard()
 {
 	int ret=0;
+	CharDBEntry* entry;
 
 	slotStatusMsg(i18n("Connection Wizard..."));
 
@@ -1057,11 +1131,13 @@ void KmudApp::slotMudConnectionWizard()
 */
 				} // end if mud->isConnected()
 				break;
-			case 2: 
+			case 2:
 				if ( !mud->isConnected() )
 				{
 					if ((d.getLogin()!="") && (d.getMudTitle()!=""))
 					{
+						entry = doc->getEntry(d.getLogin(),d.getMudTitle());
+						mudstream->setCharDBEntry(entry);
 						doc->setCurrentMudID(d.getSelectedMudID());
 						doc->setCurrentCharacterID(d.getSelectedCharacterID());
 					}
@@ -1071,7 +1147,7 @@ void KmudApp::slotMudConnectionWizard()
 						doc->setCurrentMudID("");
 					}
 					refreshEnablesDisables();
-          refreshLoggingToolbarIcon();
+					refreshLoggingToolbarIcon();
 
 					if ( (d.getLogin()!="") && (d.getMudTitle()!="") )
 					{
@@ -1119,7 +1195,7 @@ void KmudApp::slotMudClose()
 	slotStatusMsg(i18n("Closing connection..."));
 	
 	QObject::disconnect( mud, SIGNAL(filledBuffer(QString)), mudstream, SLOT(slotProcessOutput(QString)) );
-	// QObject::disconnect( view, SIGNAL(enteredInput(QString)), mudstream, SLOT(slotProcessInput(QString)) );
+	//QObject::disconnect( view, SIGNAL(enteredInput(QString)), mudstream, SLOT(slotProcessInput(QString)) );
  	QObject::disconnect( multiLineInput,SIGNAL(sendText(QString)),mudstream, SLOT(slotProcessInput(QString)) );
 	QObject::disconnect( mud, SIGNAL(changedEcho(bool)), view, SLOT(setLocalEcho(bool)) );
 	QObject::disconnect( mud, SIGNAL(changedEcho(bool)), view, SLOT(setInputEcho(bool)) );
@@ -1133,19 +1209,17 @@ void KmudApp::slotMudClose()
 
 	mudlog->close();
 
-#ifdef MAPPER
 	if (mapper)
 	{
 		slotCloseMapper();
 	}
-#endif
 
 	mudstream->reset ();
 
-	if (toolBar()->getButton(ID_MUD_CLOSE)!=NULL)
+	if (mainToolBar->getButton(ID_MUD_CLOSE)!=NULL)
 	{
-		toolBar()->removeItem(ID_MUD_CLOSE);
-		toolBar()->insertButton(Icon("kmud_connectionwizard.xpm"), ID_MUD_CONNECTION_WIZARD, true, i18n("Connect"),toolBarConnectButtonPosition);
+		mainToolBar->removeItem(ID_MUD_CLOSE);
+		mainToolBar->insertButton(Icon("kmud_connectionwizard.xpm"), ID_MUD_CONNECTION_WIZARD, true, i18n("Connect"),toolBarConnectButtonPosition);
 	}
 
 	doc->setCurrentCharacterID("");
@@ -1168,6 +1242,8 @@ void KmudApp::slotMudClose()
 	mud_menu->setItemEnabled(ID_MUD_QUICKCONNECT3, true);
 	mud_menu->setItemEnabled(ID_MUD_QUICKCONNECT4, true);
 
+	updateCustomToolbar();
+	
 	if (autoReconnect && autoReconnectOption)
 	{
 		DlgReconnect d(this,"ReconnectDialog");
@@ -1225,6 +1301,16 @@ void KmudApp::slotViewToolBar()
 
 }
 
+void KmudApp::slotViewCustomToolBar()
+{
+	bViewCustomToolbar=!bViewCustomToolbar;
+	menuBar()->setItemChecked(ID_VIEW_CUSTOM_TOOLBAR, bViewCustomToolbar);
+	enableToolBar(KToolBar::Toggle,1);
+	
+	updateCustomToolbar();
+	slotStatusMsg(i18n("Ready."));	
+}
+
 void KmudApp::slotViewStatusBar()
 {
 	///////////////////////////////////////////////////////////////////
@@ -1278,8 +1364,6 @@ void KmudApp::slotViewMultiLineInputClose()
 
 void KmudApp::slotViewMapper()
 {
-#ifdef MAPPER
-
 	if (doc->getCurrentMudID() == "")
 	{
 		QMessageBox::information(this, i18n("Error"), i18n("There is no active MUD.  Please use\nthe Connection Wizard to select one."));
@@ -1290,12 +1374,13 @@ void KmudApp::slotViewMapper()
 	//show the auto mapper
 	if (!mapper)
 	{	
-		mapper = new CMapWindow(lastTitle,lastLogin,doc);
+		mapper = new CMapWindow(lastTitle,lastLogin,doc,view);
 		mudstream->setMapper(mapper);
 		view_menu->setItemChecked(ID_VIEW_MAPPER,true);
-		// toolBar()->setToggle(ID_VIEW_MAPPER,true);
+
 		connect( mapper,SIGNAL(slotMapperClosed()),SLOT(slotCloseMapper()) );
-		connect( mapper,SIGNAL(movePlayer(direction)),SLOT(slotMovePlayer(direction)));
+		connect( mapper,SIGNAL(movePlayer(QString)),SLOT(slotMovePlayer(QString)));
+  	accelerators->connectItem("map_undo", mapper, SLOT(slotUndoAutoCreation()));
 	}
 	else
 	{
@@ -1303,62 +1388,29 @@ void KmudApp::slotViewMapper()
 	}
 
 	slotStatusMsg(i18n("Ready."));
-#endif
 }
 
 
-void KmudApp::slotMovePlayer(direction dir)
+void KmudApp::slotMovePlayer(QString dirCmd)
 {
-#ifdef MAPPER
-	directions *dirs;
-	QString moveDir;
-	
-	moveDir = "";
-	
-	CharDBEntry* entry=doc->getEntry(lastLogin,lastTitle);
-	if (entry!=NULL)
-	{
-		CMudProfile* mudPro = doc->getMudProfile(entry->mudfile);
-	 	dirs = mudPro->getDirections();
-	
-	 	switch (dir)
-	 	{
-	 		case NORTH : moveDir = dirs->north; break;
-	 		case NORTHEAST : moveDir = dirs->northeast; break;
-	 		case EAST : moveDir = dirs->east; break;
-	 		case SOUTHEAST : moveDir = dirs->southeast; break;
-	 		case SOUTH : moveDir = dirs->south; break;
-	 		case SOUTHWEST : moveDir = dirs->southwest; break;
-	 		case WEST : moveDir = dirs->west; break;
-	 		case NORTHWEST : moveDir = dirs->northwest; break;
-	 		case UP : moveDir = dirs->up; break;
-	 		case DOWN : moveDir = dirs->down; break;
-	 	}
-	 	
-	 	moveDir = moveDir + "\n";
-	 	
-	 	mud->sendData(moveDir,moveDir.length());
-	 	view->slotAppendToOutput (moveDir);
-	}
+	QString moveDir = dirCmd;
+	view->slotInputReturnPressed(moveDir);
 
-#endif
 }
 
 
 void KmudApp::slotCloseMapper(void)
 {
-#ifdef MAPPER
 	if (mapper)
 	{
+  	accelerators->disconnectItem("map_undo", mapper, SLOT(slotUndoAutoCreation()));
 		mapper->saveOptions();
 		mapper->disconnect();
 		mapper->close(true);
 		mapper = NULL;
 		mudstream->setMapper(NULL);
 		view_menu->setItemChecked(ID_VIEW_MAPPER,false);
-		// toolBar()->setToggle(ID_VIEW_MAPPER,false);
 	}
-#endif
 }
 
 
@@ -1370,12 +1422,12 @@ void KmudApp::slotOptionFont()
 
 	readOptions();
 
-	d.setOutputFont(view->getOutputFont());
+	d.setOutputFont(view->getOutputDefaultFont());
 	d.setInputFont(view->getInputFont());
 
 	if (d.exec())
 	{
-		view->setOutputFont(d.getOutputFont());
+		view->setOutputDefaultFont(d.getOutputFont());
 		mudFont=d.getOutputFont();
 		view->setInputFont(d.getInputFont());
 		inputFont=d.getInputFont();
@@ -1393,8 +1445,8 @@ void KmudApp::slotOptionColors()
 	DlgOptionColors d(this,"ColorSetupDialog", interpretType);
 
 	readOptions();
-	d.setForegroundColor(view->getOutputForegroundColor());
-	d.setBackgroundColor(view->getOutputBackgroundColor());
+	d.setForegroundColor(view->getOutputDefaultForegroundColor());
+	d.setBackgroundColor(view->getOutputDefaultBackgroundColor());
 	d.setLocalEchoColor(view->getLocalEchoColor());
 
 	d.setDebugHighlightColor(view->getDebugHighlightColor());
@@ -1415,8 +1467,8 @@ void KmudApp::slotOptionColors()
 		mudBackgroundColor = d.getBackgroundColor();
 		mudLocalEchoColor = d.getLocalEchoColor();
 		mudDebugHighlightColor = d.getDebugHighlightColor();
-		view->setOutputForegroundColor(mudForegroundColor);
-		view->setOutputBackgroundColor(mudBackgroundColor);
+		view->setOutputDefaultForegroundColor(mudForegroundColor);
+		view->setOutputDefaultBackgroundColor(mudBackgroundColor);
 		view->setInputBackgroundColor(mudBackgroundColor);
 		view->setLocalEchoColor(mudLocalEchoColor);
 		view->setDebugHighlightColor(mudDebugHighlightColor);
@@ -1468,9 +1520,12 @@ void KmudApp::slotOptionPreferences()
 	d.setAutoReconnect(autoReconnectOption);
 	d.setKeepLastInput(keepLastInput);
 	d.setLocalEchoMode(localEchoMode);
+	d.setLocalEchoSuppressNewline(localEchoSuppressNewline);
 	d.setInputEchoMode(inputEchoMode);
 	d.setOutputSize(maxOutputLines);
 	d.setInputSize(maxInputLines);
+	d.setSharedInputHistory(sharedInputHistory);
+	d.setEnableSpecialChars(enableSpecialChars);
 	d.setSeparator(separator);
 	d.setSpeedwalkingChar (speedwalkingChar);
 	d.setExternChar (externChar);
@@ -1482,16 +1537,20 @@ void KmudApp::slotOptionPreferences()
 	d.setOutputNotifyOn(mudOutputNotifyOn);
 	d.setOutputNotifySeconds(mudOutputNotifyWaitSeconds);
 	d.setConnectTimeoutSeconds(connecttimeout);
-
+  d.setPingData(doc->getPingExecutable(), doc->getPingEnabled());
+	
 	if (d.exec())
 	{
 		localEchoMode=d.getLocalEchoMode();
+		localEchoSuppressNewline=d.getLocalEchoSuppressNewline();
 		inputEchoMode=d.getInputEchoMode();
 		maxOutputLines=d.getOutputSize();
 		maxInputLines=d.getInputSize();
+		sharedInputHistory=d.getSharedInputHistory();
 		keepLastInput=d.getKeepLastInput();
 		selectAllOnFocus=d.getSelectAllOnFocus();
 		autoReconnectOption=d.getAutoReconnect();
+		enableSpecialChars=d.getEnableSpecialChars();
 		separator=d.getSeparator();
 		speedwalkingChar = d.getSpeedwalkingChar ();
 		externChar = d.getExternChar ();
@@ -1504,20 +1563,27 @@ void KmudApp::slotOptionPreferences()
 		connecttimeout=d.getConnectTimeoutSeconds();	
 
 		view->setLocalEchoMode(localEchoMode);
+		view->setLocalEchoSupressNewline(localEchoSuppressNewline);
 		view->setInputEchoMode(inputEchoMode);
 		view->setMaxHistoryLines(maxInputLines);
 		view->setMaxMudViewLines(maxOutputLines);
+		view->setSharedInputHistory(sharedInputHistory);
 		view->setKeepLastInput(keepLastInput);
 		view->setSelectAllOnFocus(selectAllOnFocus);
+		view->setEnableSpecialChars(enableSpecialChars);
 		view->setSeparator(separator);
 		view->setSpeedwalkingChar (speedwalkingChar);
 		view->setExternChar (externChar);
 		view->setUseBeep(useBeep);
 		view->setAutoTime(autoTime);
 		view->setAutoReconnect(autoReconnectOption);
+		
+		doc->setPingExecutable(d.getPingPath());
+		doc->setPingEnabled(d.getPingEnabled());
+		
 		mudOutputDoNotify = false;
 		mudOutputTimer->start(mudOutputNotifyWaitSeconds*1000,true);
-	
+			
 		saveOptions();
 	}
 	
@@ -1543,6 +1609,24 @@ void KmudApp::slotProfileAliases()
 	slotStatusMsg(i18n("Ready."));
 }
 
+void KmudApp::slotProfileToobar()
+{
+	if (doc->getCurrentCharacterID() == "") {
+		QMessageBox::information(this, i18n("Error"), i18n("There is no active profile.\nPlease connect through Connection Wizard."));
+		return;
+	}
+	slotStatusMsg(i18n("Editing custom toolbar..."));
+	
+	DlgCustomToolbar d(doc,this,"Programmable Toolbar");
+	
+	if (d.exec())
+	{
+		updateCustomToolbar();
+	}
+	
+	slotStatusMsg(i18n("Ready."));
+}
+
 void KmudApp::slotProfileTriggers()
 {
 	if (doc->getCurrentCharacterID() == "") {
@@ -1779,16 +1863,21 @@ void KmudApp::closeEvent(QCloseEvent* ce)
 void KmudApp::refreshEnablesDisables()
 {
 	bool flag=doc->getCurrentCharacterID() != "";
+	if (!flag)
+	{
+	  // no profile set, reset alias expansion and triggers
+	  doc->setAliasesDisabled(false);
+	  doc->setTriggersDisabled(false);
+	}
 	profile_menu->setItemEnabled(aliasID, flag);
 	profile_menu->setItemEnabled(triggerID, flag);
 	profile_menu->setItemEnabled(loggingID, flag);
 	profile_menu->setItemEnabled(saveID, flag);
-	toolBar()->setItemEnabled(aliasID, flag);
-	toolBar()->setItemEnabled(triggerID, flag);
-	toolBar()->setItemEnabled(ID_TOOLBAR_LOGGING_TOGGLE, flag);
-#ifdef MAPPER
-	toolBar()->setItemEnabled(ID_VIEW_MAPPER, flag);
-#endif
+	profile_menu->setItemEnabled(toolbarID, flag);	
+	mainToolBar->setItemEnabled(aliasID, flag && !doc->getAliasesDisabled());
+	mainToolBar->setItemEnabled(triggerID, flag && !doc->getTriggersDisabled());
+	mainToolBar->setItemEnabled(ID_TOOLBAR_LOGGING_TOGGLE, flag);
+	mainToolBar->setItemEnabled(ID_VIEW_MAPPER, flag);
 	
 	flag=doc->getCurrentMudID() != "";
 	option_menu->setItemEnabled(mudOptionsID, flag);
@@ -1804,9 +1893,9 @@ void KmudApp::refreshLoggingToolbarIcon()
 	if (prof!=NULL)
 	{
 		if (prof->getLoggingEnabled())
-			toolBar()->setButton(ID_TOOLBAR_LOGGING_TOGGLE,true);
+			mainToolBar->setButton(ID_TOOLBAR_LOGGING_TOGGLE,true);
 		else
-			toolBar()->setButton(ID_TOOLBAR_LOGGING_TOGGLE,false);
+			mainToolBar->setButton(ID_TOOLBAR_LOGGING_TOGGLE,false);
 	}
 
 
@@ -1818,6 +1907,18 @@ QPopupMenu* KmudApp::getPluginMenu()
 	return plugin_menu;
 }
 
+void KmudApp::slotCustomToolbarClicked(int handle)
+{
+	CCharacterProfile* profile = doc->getCharacterProfile(doc->getCurrentCharacterID());
+
+	if (profile)
+	{
+		ToolbarEntry* entry = profile->getToolbarEntryAt(handle);
+		
+		QString command = entry->command;
+		view->slotInputReturnPressed(command);
+	}
+}
 
 void KmudApp::commandCallback(int id_){
 	switch (id_){
@@ -1834,23 +1935,22 @@ void KmudApp::commandCallback(int id_){
 		ON_CMD(ID_MUD_QUICKCONNECT4,	slotMudQuickConnect(4))
 
 		ON_CMD(ID_VIEW_TOOLBAR,		slotViewToolBar())
+		ON_CMD(ID_VIEW_CUSTOM_TOOLBAR, slotViewCustomToolBar())		
 		ON_CMD(ID_VIEW_STATUSBAR,	slotViewStatusBar())
 		ON_CMD(ID_VIEW_ALTINPUT,	slotViewAltInput())
 		ON_CMD(ID_VIEW_MULTILINEINPUT,	slotViewMultiLineInput())
-#ifdef MAPPER
 		ON_CMD(ID_VIEW_MAPPER,		slotViewMapper())
-#endif
 
 		ON_CMD(ID_OPTION_FONT,		slotOptionFont())
 		ON_CMD(ID_OPTION_COLORS,	slotOptionColors())
 		ON_CMD(ID_OPTION_KEYS,    slotOptionKeys())
 		ON_CMD(ID_OPTION_PREFERENCES,	slotOptionPreferences())
 		ON_CMD(ID_OPTION_MUD,		slotOptionMud())
-		ON_CMD(ID_OPTION_SAVE,		saveOptions())
 
 		ON_CMD(ID_PROFILE_ALIASES,	slotProfileAliases())
 		ON_CMD(ID_PROFILE_TRIGGERS,	slotProfileTriggers())
 		ON_CMD(ID_PROFILE_LOGGING,	slotProfileLogging())
+		ON_CMD(ID_PROFILE_TOOLBAR,	slotProfileToobar())
 		ON_CMD(ID_PROFILE_SAVE,		slotProfileSave())
 
 		ON_CMD(ID_PLUGIN_MANAGER,		slotPluginManager())
@@ -1875,23 +1975,22 @@ void KmudApp::statusCallback(int id_){
 		ON_CMD(ID_MUD_QUICKCONNECT3,			 i18n("Connects to a previous Mud"))
 		ON_CMD(ID_MUD_QUICKCONNECT4,			 i18n("Connects to a previous Mud"))
 
-		ON_STATUS_MSG(ID_VIEW_TOOLBAR,      i18n("Enables / disables the actual Toolbar"))
+		ON_STATUS_MSG(ID_VIEW_TOOLBAR,      i18n("Enables / disables the main Toolbar"))
+		ON_STATUS_MSG(ID_VIEW_CUSTOM_TOOLBAR,      i18n("Enables / disables the programmable Toolbar"))		
 		ON_STATUS_MSG(ID_VIEW_STATUSBAR,    i18n("Enables / disables the Statusbar"))
 		ON_STATUS_MSG(ID_VIEW_ALTINPUT,     i18n("Enables / disables the alternative Input"))
 		ON_STATUS_MSG(ID_VIEW_MULTILINEINPUT,     i18n("Enables / disables the  multiline Input"))
-#ifdef MAPPER
 		ON_STATUS_MSG(ID_VIEW_MAPPER,       i18n("Invokes the auto mapper"))
-#endif
 
 		ON_STATUS_MSG(ID_OPTION_FONT,       i18n("Change font settings"))
 		ON_STATUS_MSG(ID_OPTION_COLORS,     i18n("Change color settings"))
 		ON_STATUS_MSG(ID_OPTION_KEYS,       i18n("Change accelerator keys"))
 		ON_STATUS_MSG(ID_OPTION_PREFERENCES,     i18n("Change preferences"))
 		ON_STATUS_MSG(ID_OPTION_MUD,        i18n("Change MUD Settings"))
-		ON_STATUS_MSG(ID_OPTION_SAVE,       i18n("Save options"))
 
 		ON_STATUS_MSG(ID_PROFILE_ALIASES,     i18n("Edit aliases profile"))
 		ON_STATUS_MSG(ID_PROFILE_TRIGGERS,    i18n("Edit triggers profile"))
+		ON_STATUS_MSG(ID_PROFILE_TOOLBAR,     i18n("Edit programmable toolbar profile"))
 		ON_STATUS_MSG(ID_PROFILE_LOGGING,     i18n("Customize logging"))
 		ON_STATUS_MSG(ID_PROFILE_SAVE,        i18n("Save profile"))
 
@@ -1908,6 +2007,15 @@ void KmudApp::refreshAccelerators()
 	accelerators->disconnectItem("boss_key", this, SLOT(slotHideMainWindow()));
 	accelerators->connectItem("boss_key", this, SLOT(slotHideMainWindow()));
   accelerators->changeMenuAccel(help_menu, ID_HELP_BOSS, "boss_key");
+
+  // Toggle Alias resolving
+	accelerators->disconnectItem("disable_aliases", this, SLOT(slotToggleAliasResolving()));
+	accelerators->connectItem("disable_aliases", this, SLOT(slotToggleAliasResolving()));
+
+	// Toggle Triggers
+  accelerators->disconnectItem("disable_triggers", this, SLOT(slotToggleTriggers()));
+	accelerators->connectItem("disable_triggers", this, SLOT(slotToggleTriggers()));
+
 }
 
 /** initialises key accelerators */
@@ -1920,6 +2028,24 @@ void KmudApp::initAccelerators()
 	{
 	  accelerators->insertItem(i18n("Boss key"), "boss_key", Key_Escape);
 	}
+	
+	// Toggle alias resolving, default is CTRL+A
+	if (accelerators->defaultKey("disable_aliases") == 0)
+	{
+	  accelerators->insertItem(i18n("Enable/disable Aliases"), "disable_aliases", "CTRL+A");
+	}
+	
+	// Toggle triggers on/off, default is CTRL+T
+	if (accelerators->defaultKey("disable_triggers") == 0)
+	{
+	  accelerators->insertItem(i18n("Enable/disable Triggers"), "disable_triggers", "CTRL+T");
+	}
+	
+	// Undo map change, default is CTRL-BACKSPACE
+	if (accelerators->defaultKey("map_undo") == 0)
+	{
+	  accelerators->insertItem(i18n("Mapper Undo"), "map_undo", "CTRL+Backspace");
+	}
 	// read values from kmudrc
 	accelerators->readSettings(kapp->getConfig());	
 	
@@ -1939,12 +2065,10 @@ void KmudApp::hide()
 {
   // Hide associated top level windows
 
-#ifdef MAPPER
   if (mapper != NULL)
   {
     mapper->hide();
   }
-#endif
 
   // finally use overridden method
   KTMainWindow::hide();
@@ -1955,10 +2079,22 @@ void KmudApp::show()
 {
   KTMainWindow::show();
 
-#ifdef MAPPER
   if (mapper != NULL)
   {
     mapper->show();
   }
-#endif
+}
+
+/** toggle the state of alias resolving (on/off) */
+void KmudApp::slotToggleAliasResolving()
+{
+  doc->setAliasesDisabled(!doc->getAliasesDisabled());
+  refreshEnablesDisables();
+}
+
+/** enable/disable triggers */
+void KmudApp::slotToggleTriggers()
+{
+  doc->setTriggersDisabled(!doc->getTriggersDisabled());
+  refreshEnablesDisables();
 }
