@@ -3,8 +3,8 @@
                       -------------------
     description          :
     begin                : Thu Jun 1 2000
-    copyright            : (C) 2000 by Stephan Uhlmann
-    email                : suhlmann@gmx.de
+    copyright            : (C) 2000 by Kmud Developer Team
+    email                : kmud-devel@kmud.de
  ***************************************************************************/
 
 /***************************************************************************
@@ -31,9 +31,47 @@ KmudAPI::~KmudAPI()
 }
 
 
-/*
+KmudApp* KmudAPI::getApp()
+{
+	return kmudapp;
+}
+
+KmudView* KmudAPI::getView()
+{
+	return kmudapp->getView();
+}
+
+KmudDoc* KmudAPI::getDoc()
+{
+	return kmudapp->getDoc();
+}
+
+CTelnet* KmudAPI::getConnection()
+{
+	return kmudapp->getConnection();
+}
+
+CMudLog* KmudAPI::getMudLog()
+{
+	return kmudapp->getMudLog();
+}
+
+CMapWindow* KmudAPI::getMapper()
+{
+	return kmudapp->getMapper();
+}
+
+int KmudAPI::writeToSocket(QString string)
+{
+	CTelnet *ct;
+	ct = getConnection();
+	if(!ct || !ct->isConnected()) return -1;
+	return ct->sendData(string, string.length());
+}
+
+
 QPopupMenu* KmudAPI::getPluginMenu()
 {
 	return kmudapp->getPluginMenu();
 }
-*/
+
