@@ -3,8 +3,8 @@
                       -------------------
     description          :
     begin                : Thu Jun 1 2000
-    copyright            : (C) 2000 by Stephan Uhlmann
-    email                : suhlmann@gmx.de
+    copyright            : (C) 2000 by Kmud Developer Team
+    email                : kmud-devel@kmud.de
  ***************************************************************************/
 
 /***************************************************************************
@@ -23,8 +23,12 @@
 #include <qobject.h>
 #include <qpopupmenu.h>
 
-//#include "kmud.h"
 class KmudApp;
+class KmudView;
+class KmudDoc;
+class CTelnet;
+class CMudLog;
+class CMapWindow;
 
 /**
   *@author Stephan Uhlmann
@@ -38,10 +42,18 @@ public:
 	KmudAPI(KmudApp* app);
 	~KmudAPI();
 
+	// some general get methods some important classes in Kmud
+	KmudApp* getApp();
+	KmudView* getView();
+	KmudDoc* getDoc();
+	CTelnet* getConnection();
+	CMudLog* getMudLog();
+	CMapWindow* getMapper();
+
+	// special actions
+	int writeToSocket(QString str);
+	QPopupMenu* getPluginMenu();
 
-// that's ugly. should be something like
-// bool insertToPluginMenu(blabla);
-//	QPopupMenu* getPluginMenu();
 
 private:
 	KmudApp* kmudapp;
