@@ -3,8 +3,8 @@
                       -------------------
     description          : This class docks in kpanel's tray, when the Kmud app is hidden
     begin                : Fri Jul 28 2000
-    copyright            : (C) 2000 by Kevin Krammer
-    email                : kevin.krammer@gmx.at
+    copyright            : (C) 2000 by Kmud Developer Team
+    email                : kmud-devel@kmud.de
  ***************************************************************************/
 
 /***************************************************************************
